@@ -56,6 +56,58 @@
 #include "mkdtemp.h"
 #endif
 
+/* Profiling stuff adapted from gtkfilechooserdefault */
+/* To use run:
+ *  strace -ttt -f -o logfile.txt nautilus-cd-burner
+ */
+
+#undef PROFILE_MAKE_ISO
+#ifdef PROFILE_MAKE_ISO
+
+#define PROFILE_INDENT 4
+static int profile_indent;
+
+static void
+profile_add_indent (int indent)
+{
+        profile_indent += indent;
+        if (profile_indent < 0)
+                g_error ("You screwed up your indentation");
+}
+
+static void
+_ncb_make_iso_profile_log (const char *func,
+                           int         indent,
+                           const char *msg1,
+                           const char *msg2)
+{
+        char *str;
+
+        if (indent < 0)
+                profile_add_indent (indent);
+
+        if (profile_indent == 0)
+                str = g_strdup_printf ("MARK: %s: %s %s %s", G_STRLOC, func, msg1 ? msg1 : "", msg2 ? msg2 : "");
+        else
+                str = g_strdup_printf ("MARK: %s: %*c %s %s %s", G_STRLOC, profile_indent - 1, ' ', func, msg1 ? msg1 : "", msg2 ? msg2 : "");
+
+        access (str, F_OK);
+
+        g_free (str);
+
+        if (indent > 0)
+                profile_add_indent (indent);
+}
+
+#define profile_start(x, y) _ncb_make_iso_profile_log (G_STRFUNC, PROFILE_INDENT, x, y)
+#define profile_end(x, y)   _ncb_make_iso_profile_log (G_STRFUNC, -PROFILE_INDENT, x, y)
+#define profile_msg(x, y)   _ncb_make_iso_profile_log (NULL, 0, x, y)
+#else
+#define profile_start(x, y)
+#define profile_end(x, y)
+#define profile_msg(x, y)
+#endif
+
 GQuark
 nautilus_burn_iso_error_quark (void)
 {
@@ -448,6 +500,8 @@ create_graft_file (GnomeVFSURI          *uri,
 	GnomeVFSResult           result;
 	gboolean                 stop;
 
+	profile_start ("start", NULL);
+
 	result = gnome_vfs_directory_open_from_uri (&handle,
 						    uri,
 						    GNOME_VFS_FILE_INFO_DEFAULT
@@ -518,6 +572,8 @@ create_graft_file (GnomeVFSURI          *uri,
 
 	gnome_vfs_directory_close (handle);
 	gnome_vfs_file_info_unref (info);
+
+	profile_end ("end", NULL);
 }
 
 static gboolean  
@@ -1063,6 +1119,23 @@ run_process (struct mkisofs_output    *mkisofs_output,
 	nautilus_burn_progress_set_image_spinning (FALSE);
 }
 
+static char *
+get_image_space_error_message (guint64 iso_size,
+			       guint64 space_available)
+{
+	char   *space_needed_string;
+	char   *message;
+	guint64 space_needed;
+
+	space_needed = iso_size - space_available;
+	space_needed_string = g_strdup_printf ("%" G_GINT64_FORMAT, space_needed / 1048576);
+	message = g_strdup_printf (_("The selected location does not have enough space to store the disc image (%s MiB needed)."),
+				   space_needed_string);
+	g_free (space_needed_string);
+
+	return message;
+}
+
 /**
  * nautilus_burn_make_iso:
  * @filename: name of a file to use for the image
@@ -1152,7 +1225,11 @@ nautilus_burn_make_iso (const char                  *filename,
 	
 	uri = gnome_vfs_uri_new ("burn:///");
 	state.found_file = FALSE;
+
+	profile_start ("start", "creating file list");
 	create_graft_file (uri, NULL, &state);
+	profile_end ("end", "creating file list");
+
 	gnome_vfs_uri_unref (uri);
 
 	fclose (state.graft_file);
@@ -1242,10 +1319,16 @@ nautilus_burn_make_iso (const char                  *filename,
 		g_warning ("Cannot get free space at %s\n", dirname);
 	} else if (iso_size > size) {
 		if (flags & NAUTILUS_BURN_IMAGE_CREATE_WARN_SPACE) {
+			char *message;
+
+			message = get_image_space_error_message (iso_size,
+								 size);
+
 			mkisofs_output.error = g_error_new (NAUTILUS_BURN_ISO_ERROR, 
 							    NAUTILUS_BURN_ISO_ERROR_GENERAL,
-							    _("The selected location does not have enough space to store the disc image (%ld MiB needed)."),
-							    (unsigned long)iso_size / 1048576);
+							    "%s", message);
+			g_free (message);
+
 			mkisofs_output.result = NAUTILUS_BURN_RECORDER_RESULT_ERROR;
 		} else {
 			mkisofs_output.result = NAUTILUS_BURN_RECORDER_RESULT_RETRY;
@@ -1375,7 +1458,7 @@ nautilus_burn_make_iso (const char                  *filename,
  **/
 int
 nautilus_burn_make_iso_from_drive (const char                  *filename,
-				   const NautilusBurnDrive     *drive,
+				   NautilusBurnDrive           *drive,
 				   NautilusBurnImageCreateFlags flags,
 				   NautilusBurnImageType       *type,
 				   char                       **toc_filename,
@@ -1495,10 +1578,16 @@ nautilus_burn_make_iso_from_drive (const char                  *filename,
 	} else if (iso_size > size) {
 		g_free (dirname);
 		if (flags & NAUTILUS_BURN_IMAGE_CREATE_WARN_SPACE) {
+			char *message;
+
+			message = get_image_space_error_message (iso_size,
+								 size);
+
 			mkisofs_output.error = g_error_new (NAUTILUS_BURN_ISO_ERROR, 
 							    NAUTILUS_BURN_ISO_ERROR_GENERAL,
-							    _("The selected location does not have enough space to store the disc image (%ld MiB needed)."),
-							    (unsigned long)iso_size / 1048576);
+							    "%s", message);
+			g_free (message);
+
 			mkisofs_output.result = NAUTILUS_BURN_RECORDER_RESULT_ERROR;
 		} else {
 			mkisofs_output.result = NAUTILUS_BURN_RECORDER_RESULT_RETRY;
